/*********************************************************************
 *
 * File:         common.h
 *
 * Description:  Common definitions for session and application 
 *               management API
 *
 * Author:       Tomasz Stachowiak, NPAC
 *
 * Date:         10-16-96
 *
 *********************************************************************/
#ifndef _NCS_H
#define _NCS_H

#include <stdio.h>
#include <stdlib.h>

#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <sys/time.h>

#include <netdb.h>
#include <unistd.h>

#define QUIET

#ifndef QUIET
#define PRINTF printf
#else
#define PRINTF nic 
#endif


#define MAX_UDP_SIZE 60000

#define MAX_MEMBERS 20

#define DAEMON_PORT 5000
#define MANAGER_PORT 4999
#define SERVER_PORT 4998


/*********************************************************************
 *
 * NCS Data types
 *
 *********************************************************************/
#define MSG_HEADER_LENGTH 56
#define MEMBER_LENGTH 25

typedef struct _participant
{
  unsigned char handle;
  char user[20];
  int host;
} Participant;

typedef struct _msg_header
{
  unsigned char tag;
  unsigned char session;
  Participant sender;
  Participant receiver;
  int dataLength;
} MsgHeader;

/* Tags in local messages */
typedef enum _local_tag
{
  /* local requests */
  reqLocalSessionCreate,          
  reqLocalSessionQuit,
  reqLocalSessionAddMember,
  reqLocalSessionJoin,
  reqLocalSessionReject,
  reqLocalSessionGetMembers,      
  reqLocalSessionSendToApp,
  reqLocalSessionStartApp,
  reqLocalSessionReset,

  reqLocalSessionSaveInit,
  reqLocalSessionSaveClose,
  reqLocalSessionRestoreInit,
  reqLocalSessionRestoreClose,

  /* local indications */
  indLocalSessionInvited,
  indLocalSessionAccept,
  indLocalSessionReject,          
  indLocalSessionMembers,
  indLocalSessionAddMember,
  indLocalSessionDeleteMember,
  indLocalSessionLocalMember,
  indLocalSessionStartApp,
  indLocalSessionTimeout,
  indLocalSessionConnected,

  indLocalAppQuit,
  indDbStart,
  indDbStop
} LocalTag;

typedef enum _app_tag
{
  reqAppDettach=50,
  reqAppData,
  indAppData
} AppTag;

typedef enum _new_tag
{
  reqNewLocalSessionInit=60,
  reqNewRemoteSessionConnect,
  reqNewRemoteSessionInvite,
  reqNewLocalApplicationAttach
} NewTag;


typedef enum _server_tag
{
  reqServerAddUser=100,
  reqServerDelUser,
  reqServerUsers,
  indServerUsers
} ServerTag;

typedef struct _manager_data
{
  int socket;
  struct sockaddr_in addr;
  struct sockaddr_in daemon_addr;
} ManagerData;

typedef struct _invite_data
{
  int numMembers;
  Participant parts[1];
} InviteData;

typedef struct _member_info
{
  int handle;
  Participant part;
  struct sockaddr_in addr;
} MemberInfo;

typedef struct _ncs_application
{
  int port;
  int socket;
  int ctrl_socket;
  int sessionID;
  struct sockaddr_in addr;
  struct sockaddr_in ctrl_addr;
  struct sockaddr_in daemon_addr;
  struct sockaddr_in mcast_addr;
  Participant member[MAX_MEMBERS];
  int memberActive[MAX_MEMBERS];
  int numMembers;
  int localHandle;
  void (* startingCallback)(void);
  void (* quitCallback)(void);
  void (* addMbrCallback)(int);
  void (* delMbrCallback)(int);
  void (* dataCallback)(int, int, void *, int);
} ncsApplication;


typedef struct _ncs_data
{
  int type;
  int handle;
  char data[1];
} ncsData;

typedef enum _app_clbk_tag
{
  tagAppStarting,
  tagAppQuit,
  tagAppData,
  tagAppAddMbr,
  tagAppDelMbr
} AppClbkTag;

typedef struct _NCS_participant
{
  char user[20];
  char host[50];
} NCSparticipant;

typedef struct _server_data
{
  int numMembers;
  NCSparticipant parts[1];
} ServerData;

typedef struct _app_data
{
  unsigned char handle;
  int recID;
  int port;
  short tag;
  char data[1];
} AppData;


/************************************************************************
 * 
 * NCS API functions declarations
 *
 ***********************************************************************/

/* Session managment functions */

int ncsSessionInitialize();
void ncsSessionCreate();
int  ncsSessionGetFD(fd_set *set);
void ncsProcessIncoming(fd_set *set);
void ncsSessionAddMember(char *user, char *host);
void ncsSessionAcceptInvitation(NCSparticipant *part);
void ncsSessionRejectInvitation(NCSparticipant *part);
void ncsSessionAddCallback(short tag, void (*function));
void ncsSessionQuit();
void ncsSessionQueryMembers();
void ncsSessionSendToApp(int port, short type, char *msg, int len);
void ncsSessionStartApp(int port);
void ncsSessionReset();

int ncsSessionSaveInit();
void ncsSessionSaveClose();
int ncsSessionRestoreInit();
void ncsSessionRestoreClose();



/* Applications management functions */

ncsApplication *ncsAppAttach(int port);
void ncsAppDettach(ncsApplication *app);
int ncsAppSendData(ncsApplication *app, int mbr, int type, 
		   void *data, int len);
int ncsAppSendPriorityData(ncsApplication *app, int mbr, int type, 
		   void *data, int len);
int ncsAppProcessIncoming(ncsApplication *app, fd_set *set);
int ncsAppGetData(ncsApplication *app, fd_set *set, char **data);
int ncsAppGetFD(ncsApplication *app, fd_set *set);
int ncsAppGetLocalHandle(ncsApplication *app);
int ncsAppGetNumMembers(ncsApplication *app);
void ncsAppAddCallback(ncsApplication *app, short tag, void (* function));

#endif






